
#include <stdio.h>
#include <stdlib.h>
#if !defined(_XBASICOPS_H)
#define _XBASICOPS_C
#include "Xbasicops.h"
#endif

#include "Xbascountpriv.h"


#define Flag int
#define XB_MAX_16 ((X_INT16)0x7fff)
#define XB_MIN_16 ((X_INT16)0x8000)
#define XB_MAX_32 0x7fffffffL
#define XB_MIN_32 0x80000000L

#define XB_SET_OVER Overflow = 1; nbOverFlow++

#if !defined(XB_DEBUG)
#   define __XBCTX__
#else
#   define __XBCTX__    ,file,line
#endif





/*___________________________________________________________________________
 |                                                                           |
 |   Constants and Globals                                                   |
 |___________________________________________________________________________|
*/
static Flag Overflow = 0;
static X_INT32 nbOverFlow = 0 ;
static Flag Carry = 0;

/*___________________________________________________________________________
 |                                                                           |
 |   Functions                                                               |
 |___________________________________________________________________________|
*/



static X_INT16 xb_saturate (X_INT32 L_var1 XB_DBG_ARG) {
    X_INT16 var_out;
    if (L_var1 > 0X00007fffL) {
        XB_SET_OVER ;
        var_out = XB_MAX_16;
        XBC_IncSat ; /* XBC */
    } else if (L_var1 < (X_INT32) 0xffff8000L) {
        XB_SET_OVER ;
        var_out = XB_MIN_16;
        XBC_IncSat ; /* XBC */
    }    else    {
        var_out = (X_INT16) L_var1;
    }
    return (var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_add                                                  |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |    Performs the addition (var1+var2) with overflow control and saturation;|
 |    the 16 bit result is set at +32767 when overflow occurs or at -32768   |
 |    when underflow occurs.                                                 |
 |                                                                           |
 |   Complexity weight : 1                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    var1                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the|
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |    var2                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    var_out                                                                |
 |             16 bit short signed integer (X_INT16) whose value falls in the|
 |             range : 0xffff 8000 <= var_out <= 0x0000 7fff.                |
 |___________________________________________________________________________|
*/

XB_INL  X_INT16 xb_add (X_INT16 var1, X_INT16 var2 XB_DBG_ARG) {
    X_INT16 var_out;
    X_INT32 L_sum;
    L_sum = (X_INT32) var1 + var2;

//    var_out = xb_saturate (L_sum __XBCTX__);
    if (L_sum > 0X00007fffL) {
        XB_SET_OVER ;
        var_out = XB_MAX_16;
        XBC_IncSat ; /* XBC */
    } else if (L_sum < (X_INT32) 0xffff8000L) {
        XB_SET_OVER ;
        var_out = XB_MIN_16;
        XBC_IncSat ; /* XBC */
    }    else    {
        var_out = (X_INT16) L_sum;
    }
// end  
    XBC_Incr(XBT_add) ; /* XBC */
    return (var_out);
}


/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_sub                                                  |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |    Performs the subtraction (var1+var2) with overflow control and satu-   |
 |    ration; the 16 bit result is set at +32767 when overflow occurs or at  |
 |    -32768 when underflow occurs.                                          |
 |                                                                           |
 |   Complexity weight : 1                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    var1                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the|
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |    var2                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the|
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    var_out                                                                |
 |             16 bit short signed integer (X_INT16) whose value falls in the|
 |             range : 0xffff 8000 <= var_out <= 0x0000 7fff.                |
 |___________________________________________________________________________|
*/

XB_INL X_INT16 xb_sub (X_INT16 var1, X_INT16 var2 XB_DBG_ARG){
    X_INT16 var_out;
    X_INT32 L_diff;
    L_diff = (X_INT32) var1 - var2;
//    var_out = xb_saturate (L_diff __XBCTX__);
    if (L_diff > 0X00007fffL) {
        XB_SET_OVER ;
        var_out = XB_MAX_16;
        XBC_IncSat ; /* XBC */
    } else if (L_diff < (X_INT32) 0xffff8000L) {
        XB_SET_OVER ;
        var_out = XB_MIN_16;
        XBC_IncSat ; /* XBC */
    }    else    {
        var_out = (X_INT16) L_diff;
    }
// end
    
    XBC_Incr(XBT_sub) ; /* XBC */
    return (var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_abs_s                                                |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |    Absolute value of var1; abs_s(-32768) = 32767.                         |
 |                                                                           |
 |   Complexity weight : 1                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    var1                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the|
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    var_out                                                                |
 |             16 bit short signed integer (X_INT16) whose value falls in the|
 |             range : 0x0000 0000 <= var_out <= 0x0000 7fff.                |
 |___________________________________________________________________________|
*/

XB_INL X_INT16 xb_abs_s (X_INT16 var1 XB_DBG_ARG){
    X_INT16 var_out;
    if (var1 == (X_INT16) 0X8000) {
        var_out = XB_MAX_16;
        XBC_IncSat ; /* XBC */
    }   else   {
        if (var1 < 0)  {
            var_out = (X_INT16) -var1;
        }  else     {
            var_out = var1;
        }
    }
    XBC_Incr(XBT_abs_s) ; /* XBC */
    return (var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_shl                                                  |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Arithmetically shift the 16 bit input var1 left var2 positions.Zero fill|
 |   the var2 LSB of the result. If var2 is negative, arithmetically shift   |
 |   var1 right by -var2 with sign extension. Saturate the result in case of |
 |   underflows or overflows.                                                |
 |                                                                           |
 |   Complexity weight : 1                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    var1                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |    var2                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    var_out                                                                |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var_out <= 0x0000 7fff.                |
 |___________________________________________________________________________|
*/

XB_INL X_INT16 xb_shl (X_INT16 var1, X_INT16 var2 XB_DBG_ARG){
    X_INT16 var_out;
    X_INT32 result;
    if (var2 < 0) {
//      if (var2 < -16) { var2 = -16; }
//      var_out = xb_shr (var1, (X_INT16)-var2 __XBCTX__);
//      XBC_Decr(XBT_shr) ; /* XBC */
        if (var2 <= -15) {
            var_out = (X_INT16) ((var1 < 0) ? -1 : 0) ;
        } else {
            if (var1 < 0) {
                var_out = (X_INT16) (~((~var1) >> (-var2) ));
            }  else  {
                var_out = (X_INT16) (var1 >> (-var2));
            }
        }
// end
    }  else   {
        result = (X_INT32) var1 *((X_INT32) 1 << var2);
        if ((var2 > 15 && var1 != 0) || (result != (X_INT32) ((X_INT16) result))) {
            XB_SET_OVER ;
            var_out = (X_INT16) ( (var1 > 0) ? XB_MAX_16 : XB_MIN_16 );
            XBC_IncSat ; /* XBC */
        } else  {
            var_out = xb_extract_l (result __XBCTX__);
            XBC_Decr(XBT_extract_l) ; /* XBC */
        }
    }
    XBC_Incr(XBT_shl) ; /* XBC */
    return (var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_shr                                                  |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Arithmetically shift the 16 bit input var1 right var2 positions with    |
 |   sign extension. If var2 is negative, arithmetically shift var1 left by  |
 |   -var2 with sign extension. Saturate the result in case of underflows or |
 |   overflows.                                                              |
 |                                                                           |
 |   Complexity weight : 1                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    var1                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |    var2                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    var_out                                                                |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var_out <= 0x0000 7fff.                |
 |___________________________________________________________________________|
*/

XB_INL X_INT16 xb_shr (X_INT16 var1, X_INT16 var2 XB_DBG_ARG) {
    X_INT16 var_out;
    if (var2 < 0) {
        if (var2 < -16) {  var2 = -16; }
        var_out = xb_shl (var1, (X_INT16)-var2  __XBCTX__);
        XBC_Decr(XBT_shl) ; /* XBC */
    } else {
        if (var2 >= 15) {
            var_out = (X_INT16) ((var1 < 0) ? -1 : 0) ;
        } else {
            if (var1 < 0) {
                var_out = (X_INT16) (~((~var1) >> var2));
            }  else  {
                var_out = (X_INT16) (var1 >> var2);
            }
        }
    }
    XBC_Incr(XBT_shr) ; /* XBC */
    return (var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_mult                                                 |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |    Performs the multiplication of var1 by var2 and gives a 16 bit result  |
 |    which is scaled i.e.:                                                  |
 |             mult(var1,var2) = extract_l(L_shr((var1 times var2),15)) and  |
 |             mult(-32768,-32768) = 32767.                                  |
 |                                                                           |
 |   Complexity weight : 1                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    var1                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |    var2                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    var_out                                                                |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var_out <= 0x0000 7fff.                |
 |___________________________________________________________________________|
*/

XB_INL X_INT16 xb_mult (X_INT16 var1, X_INT16 var2 XB_DBG_ARG){
    X_INT16 var_out;
    X_INT32 L_product;
    L_product = (X_INT32) var1 *(X_INT32) var2;
    L_product = (L_product & (X_INT32) 0xffff8000L) >> 15;
    if (L_product & (X_INT32) 0x00010000L) {
        L_product = L_product | (X_INT32) 0xffff0000L;
    }
//    var_out = xb_saturate (L_product __XBCTX__);
    if (L_product > 0X00007fffL) {
        XB_SET_OVER ;
        var_out = XB_MAX_16;
        XBC_IncSat ; /* XBC */
    } else if (L_product < (X_INT32) 0xffff8000L) {
        XB_SET_OVER ;
        var_out = XB_MIN_16;
        XBC_IncSat ; /* XBC */
    }    else    {
        var_out = (X_INT16) L_product;
    }

    
    XBC_Incr(XBT_mult) ; /* XBC */
    return (var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_L_mult                                               |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   L_mult is the 32 bit result of the multiplication of var1 times var2    |
 |   with one shift left i.e.:                                               |
 |        L_mult(var1,var2) = L_shl((var1 times var2),1) and                 |
 |        L_mult(-32768,-32768) = 2147483647.                                |
 |                                                                           |
 |   Complexity weight : 1                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    var1                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |    var2                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    L_var_out                                                              |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var_out <= 0x7fff ffff.              |
 |___________________________________________________________________________|
*/

XB_INL X_INT32 xb_L_mult (X_INT16 var1, X_INT16 var2 XB_DBG_ARG) {
    X_INT32 L_var_out;
    L_var_out = (X_INT32) var1 *(X_INT32) var2;
    if (L_var_out != (X_INT32) 0x40000000L) {
        L_var_out *= 2;
    }  else  {
        XB_SET_OVER ;
        L_var_out = XB_MAX_32;
        XBC_IncSat ; /* XBC */
    }
    XBC_Incr(XBT_L_mult) ; /* XBC */
    return (L_var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : L_mult_X                                                |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   L_mult_X is the 32 bit result of the multiplication of var1 times var2  |
 |   without shift left i.e.:                                                |
 |        L_mult(var1,var2) = L(var1 times var2)                             |
 |        L_mult(-32768,-32768) = 107341824.  (0x40000000)                   |
 |                                                                           |
 |   Complexity weight : 1                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    var1                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |    var2                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    L_var_out                                                              |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var_out <= 0x7fff ffff.              |
 |___________________________________________________________________________|
*/

XB_INL  X_INT32 xb_L_mult_X (X_INT16 var1, X_INT16 var2 XB_DBG_ARG) {
    X_INT32 L_var_out;
    L_var_out = (X_INT32) var1 *(X_INT32) var2;
    XBC_Incr(XBT_L_mult_X) ; /* XBC */
    return (L_var_out);
}



/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_negate                                               |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Negate var1 with saturation, saturate in the case where input is -32768:|
 |                negate(var1) = sub(0,var1).                                |
 |                                                                           |
 |   Complexity weight : 1                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    var1                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    var_out                                                                |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var_out <= 0x0000 7fff.                |
 |___________________________________________________________________________|
*/

XB_INL X_INT16 xb_negate (X_INT16 var1 XB_DBG_ARG){
    X_INT16 var_out;
    if(var1 == XB_MIN_16) {
        var_out = XB_MAX_16 ;
        XBC_IncSat ; /* XBC */
    } else {
        var_out = (X_INT16) -var1 ;
    }
    XBC_Incr(XBT_negate) ; /* XBC */
    return (var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_extract_h                                            |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Return the 16 MSB of L_var1.                                            |
 |                                                                           |
 |   Complexity weight : 1                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    L_var1                                                                 |
 |             32 bit long signed integer (X_INT32 ) whose value falls in the |
 |             range : 0x8000 0000 <= L_var1 <= 0x7fff ffff.                 |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    var_out                                                                |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var_out <= 0x0000 7fff.                |
 |___________________________________________________________________________|
*/

XB_INL X_INT16 xb_extract_h (X_INT32 L_var1 XB_DBG_ARG) {
    X_INT16 var_out;
    var_out = (X_INT16) (L_var1 >> 16);
    XBC_Incr(XBT_extract_h) ; /* XBC */
    return (var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_extract_l                                               |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Return the 16 LSB of L_var1.                                            |
 |                                                                           |
 |   Complexity weight : 1                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    L_var1                                                                 |
 |             32 bit long signed integer (X_INT32 ) whose value falls in the |
 |             range : 0x8000 0000 <= L_var1 <= 0x7fff ffff.                 |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    var_out                                                                |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var_out <= 0x0000 7fff.                |
 |___________________________________________________________________________|
*/

XB_INL X_INT16 xb_extract_l (X_INT32 L_var1 XB_DBG_ARG) {
    X_INT16 var_out;
    var_out = (X_INT16) L_var1;
    XBC_Incr(XBT_extract_l) ; /* XBC */
    return (var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_round                                                |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Round the lower 16 bits of the 32 bit input number into the MS 16 bits  |
 |   with saturation. Shift the resulting bits right by 16 and return the 16 |
 |   bit number:                                                             |
 |               round(L_var1) = extract_h(L_add(L_var1,32768))              |
 |                                                                           |
 |   Complexity weight : 1                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    L_var1                                                                 |
 |             32 bit long signed integer (X_INT32 ) whose value falls in the |
 |             range : 0x8000 0000 <= L_var1 <= 0x7fff ffff.                 |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    var_out                                                                |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var_out <= 0x0000 7fff.                |
 |___________________________________________________________________________|
*/

XB_INL X_INT16 xb_round (X_INT32 L_var1 XB_DBG_ARG) {
    X_INT16 var_out;
    X_INT32 L_rounded;
    L_rounded = xb_L_add (L_var1, (X_INT32) 0x00008000L __XBCTX__);
    XBC_Decr(XBT_L_add) ; /* XBC */
    var_out = xb_extract_h (L_rounded __XBCTX__);
    XBC_Decr(XBT_extract_h) ; /* XBC */
    XBC_Incr(XBT_round) ; /* XBC */
    return (var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_L_mac                                                |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Multiply var1 by var2 and shift the result left by 1. Add the 32 bit    |
 |   result to L_var3 with saturation, return a 32 bit result:               |
 |        L_mac(L_var3,var1,var2) = L_add(L_var3,L_mult(var1,var2)).         |
 |                                                                           |
 |   Complexity weight : 1                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    L_var3   32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var3 <= 0x7fff ffff.                 |
 |                                                                           |
 |    var1                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |    var2                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    L_var_out                                                              |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var_out <= 0x7fff ffff.              |
 |___________________________________________________________________________|
*/

XB_INL X_INT32 xb_L_mac (X_INT32 L_var3, X_INT16 var1, X_INT16 var2 XB_DBG_ARG) {
    X_INT32 L_var_out;
    X_INT32 L_product;
    L_product = xb_L_mult (var1, var2 __XBCTX__);
    XBC_Decr(XBT_L_mult) ; /* XBC */
    L_var_out = xb_L_add (L_var3, L_product __XBCTX__);
    XBC_Decr(XBT_L_add) ; /* XBC */
    XBC_Incr(XBT_L_mac) ; /* XBC */
    return (L_var_out);
}

XB_INL X_INT32 xb_L_mac_X (X_INT32 L_var3, X_INT16 var1, X_INT16 var2 XB_DBG_ARG) {
    X_INT32 L_var_out;
    X_INT32 L_product;
    L_product = xb_L_mult_X (var1, var2 __XBCTX__);
    XBC_Decr(XBT_L_mult_X) ; /* XBC */
    L_var_out = xb_L_add (L_var3, L_product __XBCTX__);
    XBC_Decr(XBT_L_add) ; /* XBC */
    XBC_Incr(XBT_L_mac_X) ; /* XBC */
    return (L_var_out);
}


/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_L_msu                                                |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Multiply var1 by var2 and shift the result left by 1. Subtract the 32   |
 |   bit result to L_var3 with saturation, return a 32 bit result:           |
 |        L_msu(L_var3,var1,var2) = L_sub(L_var3,L_mult(var1,var2)).         |
 |                                                                           |
 |   Complexity weight : 1                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    L_var3   32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var3 <= 0x7fff ffff.                 |
 |                                                                           |
 |    var1                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |    var2                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    L_var_out                                                              |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var_out <= 0x7fff ffff.              |
 |___________________________________________________________________________|
*/

XB_INL X_INT32 xb_L_msu (X_INT32 L_var3, X_INT16 var1, X_INT16 var2 XB_DBG_ARG)
{
    X_INT32 L_var_out;
    X_INT32 L_product;
    L_product = xb_L_mult (var1, var2 __XBCTX__);
    XBC_Decr(XBT_L_mult) ; /* XBC */
    L_var_out = xb_L_sub (L_var3, L_product __XBCTX__);
    XBC_Decr(XBT_L_sub) ; /* XBC */
    XBC_Incr(XBT_L_msu) ; /* XBC */
    return (L_var_out);
}

XB_INL X_INT32 xb_L_msu_X (X_INT32 L_var3, X_INT16 var1, X_INT16 var2 XB_DBG_ARG)
{
    X_INT32 L_var_out;
    X_INT32 L_product;
    L_product = xb_L_mult_X (var1, var2 __XBCTX__);
    XBC_Decr(XBT_L_mult_X) ; /* XBC */
    L_var_out = xb_L_sub (L_var3, L_product __XBCTX__);
    XBC_Decr(XBT_L_sub) ; /* XBC */
    XBC_Incr(XBT_L_msu_X) ; /* XBC */
    return (L_var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : L_macNs                                                 |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Multiply var1 by var2 and shift the result left by 1. Add the 32 bit    |
 |   result to L_var3 without saturation, return a 32 bit result. Generate   |
 |   carry and overflow values :                                             |
 |        L_macNs(L_var3,var1,var2) = L_add_c(L_var3,L_mult(var1,var2)).     |
 |                                                                           |
 |   Complexity weight : 1                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    L_var3   32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var3 <= 0x7fff ffff.                 |
 |                                                                           |
 |    var1                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |    var2                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    L_var_out                                                              |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var_out <= 0x7fff ffff.              |
 |                                                                           |
 |   Caution :                                                               |
 |                                                                           |
 |    In some cases the Carry flag has to be cleared or set before using     |
 |    operators which take into account its value.                           |
 |___________________________________________________________________________|
*/

XB_INL X_INT32 xb_L_macNs (X_INT32 L_var3, X_INT16 var1, X_INT16 var2 XB_DBG_ARG)
{
    X_INT32 L_var_out;
    L_var_out = xb_L_mult (var1, var2 __XBCTX__);
    XBC_Decr(XBT_L_mult) ; /* XBC */
    L_var_out = xb_L_add_c (L_var3, L_var_out __XBCTX__);
    XBC_Decr(XBT_L_add_c) ; /* XBC */
    XBC_Incr(XBT_L_macNs) ; /* XBC */
    return (L_var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : L_msuNs                                                 |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Multiply var1 by var2 and shift the result left by 1. Subtract the 32   |
 |   bit result from L_var3 without saturation, return a 32 bit result. Ge-  |
 |   nerate carry and overflow values :                                      |
 |        L_msuNs(L_var3,var1,var2) = L_sub_c(L_var3,L_mult(var1,var2)).     |
 |                                                                           |
 |   Complexity weight : 1                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    L_var3   32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var3 <= 0x7fff ffff.                 |
 |                                                                           |
 |    var1                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |    var2                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    L_var_out                                                              |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var_out <= 0x7fff ffff.              |
 |                                                                           |
 |   Caution :                                                               |
 |                                                                           |
 |    In some cases the Carry flag has to be cleared or set before using     |
 |    operators which take into account its value.                           |
 |___________________________________________________________________________|
*/

XB_INL X_INT32 xb_L_msuNs (X_INT32 L_var3, X_INT16 var1, X_INT16 var2 XB_DBG_ARG)
{
    X_INT32 L_var_out;
    L_var_out = xb_L_mult (var1, var2 __XBCTX__);
    XBC_Decr(XBT_L_mult) ; /* XBC */
    L_var_out = xb_L_sub_c (L_var3, L_var_out __XBCTX__);
    XBC_Decr(XBT_L_sub_c) ; /* XBC */
    XBC_Incr(XBT_L_msuNs) ; /* XBC */
    return (L_var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_L_add                                                |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   32 bits addition of the two 32 bits variables (L_var1+L_var2) with      |
 |   overflow control and saturation; the result is set at +2147483647 when  |
 |   overflow occurs or at -2147483648 when underflow occurs.                |
 |                                                                           |
 |   Complexity weight : 2                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    L_var1   32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var3 <= 0x7fff ffff.                 |
 |                                                                           |
 |    L_var2   32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var3 <= 0x7fff ffff.                 |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    L_var_out                                                              |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var_out <= 0x7fff ffff.              |
 |___________________________________________________________________________|
*/

XB_INL X_INT32 xb_L_add (X_INT32 L_var1, X_INT32 L_var2 XB_DBG_ARG) {
    X_INT32 L_var_out;
    L_var_out = L_var1 + L_var2;
    if (((L_var1 ^ L_var2) & XB_MIN_32) == 0) {
        if ((L_var_out ^ L_var1) & XB_MIN_32) {
            L_var_out = (L_var1 < 0) ? XB_MIN_32 : XB_MAX_32;
            XBC_IncSat ; /* XBC */
            XB_SET_OVER ;
        }
    }
    XBC_Incr(XBT_L_add) ; /* XBC */
    return (L_var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_L_sub                                                   |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   32 bits subtraction of the two 32 bits variables (L_var1-L_var2) with   |
 |   overflow control and saturation; the result is set at +2147483647 when  |
 |   overflow occurs or at -2147483648 when underflow occurs.                |
 |                                                                           |
 |   Complexity weight : 2                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    L_var1   32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var3 <= 0x7fff ffff.                 |
 |                                                                           |
 |    L_var2   32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var3 <= 0x7fff ffff.                 |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    L_var_out                                                              |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var_out <= 0x7fff ffff.              |
 |___________________________________________________________________________|
*/

XB_INL X_INT32 xb_L_sub (X_INT32 L_var1, X_INT32 L_var2 XB_DBG_ARG) {
    X_INT32 L_var_out;
    L_var_out = L_var1 - L_var2;
    if (((L_var1 ^ L_var2) & XB_MIN_32) != 0)    {
        if ((L_var_out ^ L_var1) & XB_MIN_32) {
            L_var_out = (L_var1 < 0L) ? XB_MIN_32 : XB_MAX_32;
            XBC_IncSat ; /* XBC */
            XB_SET_OVER ;
        }
    }
    XBC_Incr(XBT_L_sub) ; /* XBC */
   return (L_var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_L_add_c                                              |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Performs 32 bits addition of the two 32 bits variables (L_var1+L_var2+C)|
 |   with carry. No saturation. Generate carry and Overflow values. The car- |
 |   ry and overflow values are binary variables which can be tested and as- |
 |   signed values.                                                          |
 |                                                                           |
 |   Complexity weight : 2                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    L_var1   32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var3 <= 0x7fff ffff.                 |
 |                                                                           |
 |    L_var2   32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var3 <= 0x7fff ffff.                 |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    L_var_out                                                              |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var_out <= 0x7fff ffff.              |
 |                                                                           |
 |   Caution :                                                               |
 |                                                                           |
 |    In some cases the Carry flag has to be cleared or set before using     |
 |    operators which take into account its value.                           |
 |___________________________________________________________________________|
*/
XB_INL X_INT32 xb_L_add_c (X_INT32 L_var1, X_INT32 L_var2 XB_DBG_ARG){
    X_INT32 L_var_out;
    X_INT32 L_test;
    Flag carry_int = 0;
    L_var_out = L_var1 + L_var2 + Carry;
    L_test = L_var1 + L_var2;

    if ((L_var1 > 0) && (L_var2 > 0) && (L_test < 0)) {
        XB_SET_OVER ;
        carry_int = 0;
    }  else   {
        if ((L_var1 < 0) && (L_var2 < 0)) {
            if (L_test >= 0) {
                XB_SET_OVER ;
                carry_int = 1;
            }    else   {
                Overflow = 0;
                carry_int = 1;
            }
        }  else {
            if (((L_var1 ^ L_var2) < 0) && (L_test >= 0)) {
                Overflow = 0;
                carry_int = 1;
            }  else {
                Overflow = 0;
                carry_int = 0;
            }
        }
    }
    if (Carry)     {
        if (L_test == XB_MAX_32)  {
            XB_SET_OVER ;
            Carry = carry_int;
        }  else     {
            if (L_test == (X_INT32) 0xFFFFFFFFL) {
                Carry = 1;
            }   else  {
                Carry = carry_int;
            }
        }
    }  else  {
        Carry = carry_int;
    }
    XBC_Incr(XBT_L_add_c) ; /* XBC */
    return (L_var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_L_sub_c                                              |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Performs 32 bits subtraction of the two 32 bits variables with carry    |
 |   (borrow) : L_var1-L_var2-C. No saturation. Generate carry and Overflow  |
 |   values. The carry and overflow values are binary variables which can    |
 |   be tested and assigned values.                                          |
 |                                                                           |
 |   Complexity weight : 2                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    L_var1   32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var3 <= 0x7fff ffff.                 |
 |                                                                           |
 |    L_var2   32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var3 <= 0x7fff ffff.                 |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    L_var_out                                                              |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var_out <= 0x7fff ffff.              |
 |                                                                           |
 |   Caution :                                                               |
 |                                                                           |
 |    In some cases the Carry flag has to be cleared or set before using     |
 |    operators which take into account its value.                           |
 |___________________________________________________________________________|
*/

XB_INL X_INT32 xb_L_sub_c (X_INT32 L_var1, X_INT32 L_var2 XB_DBG_ARG) {
    X_INT32 L_var_out;
    X_INT32 L_test;
    Flag carry_int = 0;
    if (Carry) {
        Carry = 0;
        if (L_var2 != XB_MIN_32)  {
            L_var_out = xb_L_add_c (L_var1, -L_var2 __XBCTX__);
            XBC_Decr(XBT_L_add_c) ; /* XBC */
        } else {
            L_var_out = L_var1 - L_var2;
            if (L_var1 > 0L)  {
                XB_SET_OVER ;
                Carry = 0;
            }
        }
    } else {
        L_var_out = L_var1 - L_var2 - (X_INT32) 0X00000001L;
        L_test = L_var1 - L_var2;
        if ((L_test < 0) && (L_var1 > 0) && (L_var2 < 0))      {
            XB_SET_OVER ;
            carry_int = 0;
        } else if ((L_test > 0) && (L_var1 < 0) && (L_var2 > 0))    {
            XB_SET_OVER ;
            carry_int = 1;
        } else if ((L_test > 0) && ((L_var1 ^ L_var2) > 0))   {
            Overflow = 0;
            carry_int = 1;
        } if (L_test == XB_MIN_32)   {
            XB_SET_OVER ;
            Carry = carry_int;
        }  else  {
            Carry = carry_int;
        }
    }
    XBC_Incr(XBT_L_sub_c) ; /* XBC */
    return (L_var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_L_negate                                             |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Negate the 32 bit variable L_var1 with saturation; saturate in the case |
 |   where input is -2147483648 (0x8000 0000).                               |
 |                                                                           |
 |   Complexity weight : 2                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    L_var1   32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var3 <= 0x7fff ffff.                 |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    L_var_out                                                              |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var_out <= 0x7fff ffff.              |
 |___________________________________________________________________________|
*/

XB_INL X_INT32 xb_L_negate (X_INT32 L_var1 XB_DBG_ARG){
    X_INT32 L_var_out;
    if(L_var1 == XB_MIN_32) {
        L_var_out = XB_MAX_32 ;
        XBC_IncSat ; /* XBC */
    } else {
        L_var_out = -L_var1 ;   
    }
    XBC_Incr(XBT_L_negate) ; /* XBC */
   return (L_var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_mult_r                                               |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Same as mult with rounding, i.e.:                                       |
 |     mult_r(var1,var2) = extract_l(L_shr(((var1 * var2) + 16384),15)) and  |
 |     mult_r(-32768,-32768) = 32767.                                        |
 |                                                                           |
 |   Complexity weight : 2                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    var1                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |    var2                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    var_out                                                                |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var_out <= 0x0000 7fff.                |
 |___________________________________________________________________________|
*/

XB_INL X_INT16 xb_mult_r (X_INT16 var1, X_INT16 var2 XB_DBG_ARG) {
    X_INT16 var_out;
    X_INT32 L_product_arr;

    L_product_arr = (X_INT32) var1 *(X_INT32) var2;       /* product */
    L_product_arr += (X_INT32) 0x00004000L;      /* round */
    L_product_arr &= (X_INT32) 0xffff8000L;
    L_product_arr >>= 15;       /* shift */

    if (L_product_arr & (X_INT32) 0x00010000L)  { /* sign extend when necessary */
        L_product_arr |= (X_INT32) 0xffff0000L;
    }
//    var_out = xb_saturate (L_product_arr __XBCTX__);
    if (L_product_arr > 0X00007fffL) {
        XB_SET_OVER ;
        var_out = XB_MAX_16;
        XBC_IncSat ; /* XBC */
    } else if (L_product_arr < (X_INT32) 0xffff8000L) {
        XB_SET_OVER ;
        var_out = XB_MIN_16;
        XBC_IncSat ; /* XBC */
    }    else    {
        var_out = (X_INT16) L_product_arr;
    }

    XBC_Incr(XBT_mult_r) ; /* XBC */
    return (var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_L_shl                                                |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Arithmetically shift the 32 bit input L_var1 left var2 positions. Zero  |
 |   fill the var2 LSB of the result. If var2 is negative, arithmetically    |
 |   shift L_var1 right by -var2 with sign extension. Saturate the result in |
 |   case of underflows or overflows.                                        |
 |                                                                           |
 |   Complexity weight : 2                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    L_var1   32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var3 <= 0x7fff ffff.                 |
 |                                                                           |
 |    var2                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    L_var_out                                                              |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var_out <= 0x7fff ffff.              |
 |___________________________________________________________________________|
*/

XB_INL X_INT32 xb_L_shl (X_INT32 L_var1, X_INT16 var2 XB_DBG_ARG) {
    X_INT32 L_var_out;
    if (var2 <= 0) {
        if (var2 < -32){   var2 = -32; }
        L_var_out = xb_L_shr (L_var1, (X_INT16) -var2 __XBCTX__);
        XBC_Decr(XBT_L_shr) ; /* XBC */
    }  else   {
        for (; var2 > 0; var2--)   {
            if (L_var1 > (X_INT32) 0X3fffffffL)      {
                XB_SET_OVER ;
                XBC_IncSat ; /* XBC */
                L_var_out = XB_MAX_32;
                break;
            }    else   {
                if (L_var1 < (X_INT32) 0xc0000000L)  {
                    XB_SET_OVER ;
                    XBC_IncSat ; /* XBC */
                    L_var_out = XB_MIN_32;
                    break;
                }
            }
            L_var1 *= 2;
            L_var_out = L_var1;
        }
    }
    XBC_Incr(XBT_L_shl) ; /* XBC */
    return (L_var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : L_shr                                                   |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Arithmetically shift the 32 bit input L_var1 right var2 positions with  |
 |   sign extension. If var2 is negative, arithmetically shift L_var1 left   |
 |   by -var2 and zero fill the -var2 LSB of the result. Saturate the result |
 |   in case of underflows or overflows.                                     |
 |                                                                           |
 |   Complexity weight : 2                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    L_var1   32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var3 <= 0x7fff ffff.                 |
 |                                                                           |
 |    var2                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    L_var_out                                                              |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var_out <= 0x7fff ffff.              |
 |___________________________________________________________________________|
*/

XB_INL X_INT32 xb_L_shr (X_INT32 L_var1, X_INT16 var2 XB_DBG_ARG) {
    X_INT32 L_var_out;
    if (var2 < 0) {
        if (var2 < -32) {  var2 = (X_INT16) -32; }
        L_var_out = xb_L_shl (L_var1, (X_INT16) -var2 __XBCTX__);
        XBC_Decr(XBT_L_shl) ; /* XBC */
    } else {
        if (var2 >= 31) {
            L_var_out = (L_var1 < 0L) ? -1 : 0;
        }  else  {
            if (L_var1 < 0)  {
                L_var_out = ~((~L_var1) >> var2);
            } else {
                L_var_out = L_var1 >> var2;
            }
        }
    }
    XBC_Incr(XBT_L_shr) ; /* XBC */
    return (L_var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_shr_r                                                |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Same as shr(var1,var2) but with rounding. Saturate the result in case of|
 |   underflows or overflows :                                               |
 |    - If var2 is greater than zero :                                       |
 |          if (sub(shl(shr(var1,var2),1),shr(var1,sub(var2,1))))            |
 |          is equal to zero                                                 |
 |                     then                                                  |
 |                     shr_r(var1,var2) = shr(var1,var2)                     |
 |                     else                                                  |
 |                     shr_r(var1,var2) = add(shr(var1,var2),1)              |
 |    - If var2 is less than or equal to zero :                              |
 |                     shr_r(var1,var2) = shr(var1,var2).                    |
 |                                                                           |
 |   Complexity weight : 2                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    var1                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |    var2                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    var_out                                                                |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var_out <= 0x0000 7fff.                |
 |___________________________________________________________________________|
*/

XB_INL X_INT16 xb_shr_r (X_INT16 var1, X_INT16 var2 XB_DBG_ARG) {
    X_INT16 var_out;
    if (var2 > 15){
        var_out = 0;
    } else  {
        var_out = xb_shr (var1, var2 __XBCTX__);
        XBC_Decr(XBT_shr) ; /* XBC */
        if (var2 > 0)    {
            if ((var1 & ((X_INT16) 1 << (var2 - 1))) != 0){
                var_out++;
            }
        }
    }
    XBC_Incr(XBT_shr_r) ; /* XBC */
    return (var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_mac_r                                                   |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Multiply var1 by var2 and shift the result left by 1. Add the 32 bit    |
 |   result to L_var3 with saturation. Round the LS 16 bits of the result    |
 |   into the MS 16 bits with saturation and shift the result right by 16.   |
 |   Return a 16 bit result.                                                 |
 |            mac_r(L_var3,var1,var2) = round(L_mac(L_var3,var1,var2))       |
 |                                                                           |
 |   Complexity weight : 2                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    L_var3   32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var3 <= 0x7fff ffff.                 |
 |                                                                           |
 |    var1                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |    var2                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    var_out                                                                |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0x0000 8000 <= L_var_out <= 0x0000 7fff.              |
 |___________________________________________________________________________|
*/

XB_INL X_INT16 xb_mac_r (X_INT32 L_var3, X_INT16 var1, X_INT16 var2 XB_DBG_ARG)
{
    X_INT16 var_out;
    L_var3 = xb_L_mac (L_var3, var1, var2 __XBCTX__ );
    XBC_Decr(XBT_L_mac) ; /* XBC */
    L_var3 = xb_L_add (L_var3, (X_INT32) 0x00008000L __XBCTX__);
    XBC_Decr(XBT_L_add) ; /* XBC */
    var_out = xb_extract_h (L_var3 __XBCTX__);
    XBC_Decr(XBT_extract_h) ; /* XBC */
    XBC_Incr(XBT_mac_r) ; /* XBC */
    return (var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : msu_r                                                   |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Multiply var1 by var2 and shift the result left by 1. Subtract the 32   |
 |   bit result to L_var3 with saturation. Round the LS 16 bits of the res-  |
 |   ult into the MS 16 bits with saturation and shift the result right by   |
 |   16. Return a 16 bit result.                                             |
 |            msu_r(L_var3,var1,var2) = round(L_msu(L_var3,var1,var2))       |
 |                                                                           |
 |   Complexity weight : 2                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    L_var3   32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= L_var3 <= 0x7fff ffff.                 |
 |                                                                           |
 |    var1                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |    var2                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    var_out                                                                |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0x0000 8000 <= L_var_out <= 0x0000 7fff.              |
 |___________________________________________________________________________|
*/

XB_INL X_INT16 xb_msu_r (X_INT32 L_var3, X_INT16 var1, X_INT16 var2 XB_DBG_ARG){
    X_INT16 var_out;
    L_var3 = xb_L_msu (L_var3, var1, var2 __XBCTX__);
    XBC_Decr(XBT_L_msu) ; /* XBC */
    L_var3 = xb_L_add (L_var3, (X_INT32) 0x00008000L __XBCTX__);
    XBC_Decr(XBT_L_add) ; /* XBC */
    var_out = xb_extract_h (L_var3 __XBCTX__);
    XBC_Decr(XBT_extract_h) ; /* XBC */
    XBC_Incr(XBT_msu_r) ; /* XBC */
    return (var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : L_deposit_h                                             |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Deposit the 16 bit var1 into the 16 MS bits of the 32 bit output. The   |
 |   16 LS bits of the output are zeroed.                                    |
 |                                                                           |
 |   Complexity weight : 2                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    var1                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    L_var_out                                                              |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= var_out <= 0x7fff 0000.                |
 |___________________________________________________________________________|
*/

XB_INL X_INT32 xb_L_deposit_h (X_INT16 var1 XB_DBG_ARG) {
    X_INT32 L_var_out;
    L_var_out = (X_INT32) var1 << 16;
    XBC_Incr(XBT_L_deposit_h) ; /* XBC */
    return (L_var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : L_deposit_l                                             |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Deposit the 16 bit var1 into the 16 LS bits of the 32 bit output. The   |
 |   16 MS bits of the output are sign extended.                             |
 |                                                                           |
 |   Complexity weight : 2                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    var1                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    L_var_out                                                              |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0xFFFF 8000 <= var_out <= 0x0000 7fff.                |
 |___________________________________________________________________________|
*/

XB_INL X_INT32 xb_L_deposit_l (X_INT16 var1 XB_DBG_ARG) {
    X_INT32 L_var_out;
    L_var_out = (X_INT32) var1;
    XBC_Incr(XBT_L_deposit_l) ; /* XBC */
    return (L_var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : L_shr_r                                                 |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Same as L_shr(L_var1,var2) but with rounding. Saturate the result in    |
 |   case of underflows or overflows :                                       |
 |    - If var2 is greater than zero :                                       |
 |          if (L_sub(L_shl(L_shr(L_var1,var2),1),L_shr(L_var1,sub(var2,1))))|
 |          is equal to zero                                                 |
 |                     then                                                  |
 |                     L_shr_r(L_var1,var2) = L_shr(L_var1,var2)             |
 |                     else                                                  |
 |                     L_shr_r(L_var1,var2) = L_add(L_shr(L_var1,var2),1)    |
 |    - If var2 is less than or equal to zero :                              |
 |                     L_shr_r(L_var1,var2) = L_shr(L_var1,var2).            |
 |                                                                           |
 |   Complexity weight : 3                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    L_var1                                                                 |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= var1 <= 0x7fff ffff.                   |
 |                                                                           |
 |    var2                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    L_var_out                                                              |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= var_out <= 0x7fff ffff.                |
 |___________________________________________________________________________|
*/

XB_INL X_INT32 xb_L_shr_r (X_INT32 L_var1, X_INT16 var2 XB_DBG_ARG) {
    X_INT32 L_var_out;
    if (var2 > 31)  {
        L_var_out = 0;
    } else   {
        L_var_out = xb_L_shr (L_var1, var2 __XBCTX__ );
        XBC_Decr(XBT_L_shr) ; /* XBC */
        if (var2 > 0)  {
            if ((L_var1 & ((X_INT32) 1 << (var2 - 1))) != 0)   {
                L_var_out++;
            }
        }
    }
    XBC_Incr(XBT_L_shr_r) ; /* XBC */
    return (L_var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_L_abs                                                   |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |    Absolute value of L_var1; Saturate in case where the input is          |
 |                                                               -214783648  |
 |                                                                           |
 |   Complexity weight : 3                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    L_var1                                                                 |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= var1 <= 0x7fff ffff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    L_var_out                                                              |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x0000 0000 <= var_out <= 0x7fff ffff.                |
 |___________________________________________________________________________|
*/

XB_INL X_INT32 xb_L_abs (X_INT32 L_var1 XB_DBG_ARG) {
    X_INT32 L_var_out;
    if (L_var1 == XB_MIN_32) {
        L_var_out = XB_MAX_32;
        XBC_IncSat ; /* XBC */
    } else {
        if (L_var1 < 0)  {
            L_var_out = -L_var1;
        } else   {
            L_var_out = L_var1;
        }
    }
    XBC_Incr(XBT_L_abs) ; /* XBC */
    return (L_var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : L_sat                                                   |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |    32 bit L_var1 is set to 2147483647 if an overflow occured or to        |
 |    -2147483648 if an underflow occured on the most recent L_add_c,        |
 |    L_sub_c, L_macNs or L_msuNs operations. The carry and overflow values  |
 |    are binary values which can be tested and assigned values.             |
 |                                                                           |
 |   Complexity weight : 4                                                   |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    L_var1                                                                 |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= var1 <= 0x7fff ffff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    L_var_out                                                              |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= var_out <= 0x7fff ffff.                |
 |___________________________________________________________________________|
*/

XB_INL X_INT32 xb_L_sat (X_INT32 L_var1 XB_DBG_ARG) {
    X_INT32 L_var_out;
    L_var_out = L_var1;
    if (Overflow) {
        XBC_IncSat ; /* XBC */
        if (Carry) {
            L_var_out = XB_MIN_32;
        } else  {
            L_var_out = XB_MAX_32;
        }
        Carry = 0;
        Overflow = 0;
    }
    XBC_Incr(XBT_L_sat) ; /* XBC */
    return (L_var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : norm_s                                                  |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Produces the number of left shift needed to normalize the 16 bit varia- |
 |   ble var1 for positive values on the interval with minimum of 16384 and  |
 |   maximum of 32767, and for negative values on the interval with minimum  |
 |   of -32768 and maximum of -16384; in order to normalize the result, the  |
 |   following operation must be done :                                      |
 |                    norm_var1 = shl(var1,norm_s(var1)).                    |
 |                                                                           |
 |   Complexity weight : 15                                                  |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    var1                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0xffff 8000 <= var1 <= 0x0000 7fff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    var_out                                                                |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0x0000 0000 <= var_out <= 0x0000 000f.                |
 |___________________________________________________________________________|
*/

XB_INL X_INT16 xb_norm_s (X_INT16 var1 XB_DBG_ARG) {
    X_INT16 var_out;
    if (var1 == 0) {
        var_out = 0;
    }  else  {
        if (var1 == (X_INT16) 0xffff)    {
            var_out = 15;
        }  else {
            if (var1 < 0)   {
                var1 = (X_INT16) ~var1;
            }
            for (var_out = 0; var1 < 0x4000; var_out++)  {
                var1 <<= 1;
            }
        }
    }
    XBC_Incr(XBT_norm_s) ; /* XBC */
    return (var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : xb_div_s                                                |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Produces a result which is the fractional integer division of var1  by  |
 |   var2; var1 and var2 must be positive and var2 must be greater or equal  |
 |   to var1; the result is positive (leading bit equal to 0) and truncated  |
 |   to 16 bits.                                                             |
 |   If var1 = var2 then div(var1,var2) = 32767.                             |
 |                                                                           |
 |   Complexity weight : 18                                                  |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    var1                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0x0000 0000 <= var1 <= var2 and var2 != 0.            |
 |                                                                           |
 |    var2                                                                   |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : var1 <= var2 <= 0x0000 7fff and var2 != 0.            |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    var_out                                                                |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0x0000 0000 <= var_out <= 0x0000 7fff.                |
 |             It's a Q15 value (point between b15 and b14).                 |
 |___________________________________________________________________________|
*/

XB_INL X_INT16 xb_div_s (X_INT16 var1, X_INT16 var2 XB_DBG_ARG) {
    X_INT16 var_out = 0;
    X_INT16 iteration;
    X_INT32 L_num;
    X_INT32 L_denom;

    if ((var1 > var2) || (var1 < 0) || (var2 < 0)) {
        printf ("Division Error var1=%d  var2=%d\n", var1, var2);
        abort(); /* exit (0); */
    }
    if (var2 == 0)  {
        printf ("Division by 0, Fatal error \n");
        abort(); /* exit (0); */
    }
    if (var1 == 0)  {
        var_out = 0;
    } else {
        if (var1 == var2)     {
            var_out = XB_MAX_16;
        } else {
            L_num = xb_L_deposit_l (var1 __XBCTX__);
            XBC_Decr(XBT_L_deposit_l) ; /* XBC */
            L_denom = xb_L_deposit_l (var2 __XBCTX__);
            XBC_Decr(XBT_L_deposit_l) ; /* XBC */
            for (iteration = 0; iteration < 15; iteration++)    {
                var_out <<= 1;
                L_num <<= 1;
                if (L_num >= L_denom){
// A verifier       L_num = xb_L_sub (L_num, L_denom);
//                  XBC_Decr(XBT_L_sub) ; /* XBC */
                    L_num = L_num - L_denom ;
// a verifier         var_out = xb_add (var_out, 1);
//                    XBC_Decr(XBT_add) ; /* XBC */
                    var_out++ ;
                }
            }
        }
    }
    XBC_Incr(XBT_div_s) ; /* XBC */
    return (var_out);
}

/*___________________________________________________________________________
 |                                                                           |
 |   Function Name : norm_l                                                  |
 |                                                                           |
 |   Purpose :                                                               |
 |                                                                           |
 |   Produces the number of left shifts needed to normalize the 32 bit varia-|
 |   ble L_var1 for positive values on the interval with minimum of          |
 |   1073741824 and maximum of 2147483647, and for negative values on the in-|
 |   terval with minimum of -2147483648 and maximum of -1073741824; in order |
 |   to normalize the result, the following operation must be done :         |
 |                   norm_L_var1 = L_shl(L_var1,norm_l(L_var1)).             |
 |                                                                           |
 |   Complexity weight : 30                                                  |
 |                                                                           |
 |   Inputs :                                                                |
 |                                                                           |
 |    L_var1                                                                 |
 |             32 bit long signed integer (X_INT32) whose value falls in the  |
 |             range : 0x8000 0000 <= var1 <= 0x7fff ffff.                   |
 |                                                                           |
 |   Outputs :                                                               |
 |                                                                           |
 |    none                                                                   |
 |                                                                           |
 |   Return Value :                                                          |
 |                                                                           |
 |    var_out                                                                |
 |             16 bit short signed integer (X_INT16) whose value falls in the |
 |             range : 0x0000 0000 <= var_out <= 0x0000 001f.                |
 |___________________________________________________________________________|
*/

XB_INL X_INT16 xb_norm_l (X_INT32 L_var1 XB_DBG_ARG) {
    X_INT16 var_out;
    if (L_var1 == 0){
        var_out = 0;
    } else   {
        if (L_var1 == (X_INT32) 0xffffffffL)   {
            var_out = 31;
        }   else {
            if (L_var1 < 0) {
                L_var1 = ~L_var1;
            }
            for (var_out = 0; L_var1 < (X_INT32) 0x40000000L; var_out++)  {
                L_var1 <<= 1;
            }
        }
    }
    XBC_Incr(XBT_norm_l) ; /* XBC */
    return (var_out);
}



XB_INL X_INT16 xb_and(X_INT16 var1,X_INT16 var2 XB_DBG_ARG) {
    XBC_Incr(XBT_and) ;
    return (var1 & var2) ;
}
XB_INL X_INT16 xb_or(X_INT16 var1,X_INT16 var2 XB_DBG_ARG) {
    XBC_Incr(XBT_or) ;
    return (var1 | var2) ;
}
XB_INL X_INT16 xb_xor(X_INT16 var1,X_INT16 var2 XB_DBG_ARG) {
    XBC_Incr(XBT_xor) ;
    return (var1 ^ var2) ;
}
XB_INL X_INT32 xb_L_and(X_INT32 var1,X_INT32 var2 XB_DBG_ARG) {
    XBC_Incr(XBT_L_and) ;
    return (var1 & var2) ;
}
XB_INL X_INT32 xb_L_or(X_INT32 var1,X_INT32 var2 XB_DBG_ARG) {
    XBC_Incr(XBT_L_or) ;
    return (var1 | var2) ;
}
XB_INL X_INT32 xb_L_xor(X_INT32 var1,X_INT32 var2 XB_DBG_ARG) {
    XBC_Incr(XBT_L_xor) ;
    return (var1 ^ var2) ;
}


XB_INL void xb_Move16( XB_DBG_ARG_VOID) {
    XBC_Incr(XBT_DataMove16) ; /* XBC */
}
XB_INL void xb_Move32( XB_DBG_ARG_VOID) {
    XBC_Incr(XBT_DataMove32) ; /* XBC */
}
XB_INL void xb_Logic16( XB_DBG_ARG_VOID) {
    XBC_Incr(XBT_Logic16) ; /* XBC */
}
XB_INL void xb_Logic32( XB_DBG_ARG_VOID) {
    XBC_Incr(XBT_Logic32) ; /* XBC */
}
XB_INL void xb_Test( XB_DBG_ARG_VOID) {
    XBC_Incr(XBT_Test) ; /* XBC */
}


XB_INL void xb_set_carry(X_INT16 newC ) {
    Carry =newC & 1 ;
}
XB_INL X_INT16 xb_get_carry(void) {
    return Carry ;
}
    
XB_INL void xb_set_overflow(X_INT16 newOvF ) {
    Overflow = newOvF & 1 ;
}
XB_INL X_INT16 xb_get_Overflow(void) {
    return Overflow ;
}
