/*=======================================================================================================*/
/*      Distributed Speech Recognition                                                                   */
/*      ETSI ES 202 050 Advanced Front-End Feature Extraction Algorithm & Compression Algorithm &        */
/*      ETSI ES 202 212 Extended Advanced Front-End Feature Extraction Algorithm & Compression Algorithm */  
/*      Bit exact C-language software implementation using ETSI basic operations library                 */
/*      3GPP TS 26.243 v 6.1.0 November 2004                                                                  */
/*      ANSI C code for the Fixed-Point Distributed Speech Recognition Extended Advanced Front-end       */
/*=======================================================================================================*/
#ifndef _16kHzPROCESSING_F_H
#define _16kHzPROCESSING_F_H

#define HP16k_MEL_ORDER      5
#define HP16k_MEL_USED       3   /* must be (HP16k_MEL_ORDER - 2) because both first and last halfbands are not used! */
#define USE_INTERP_BAND      0   /* use an interpolation band between bands 23 and 24 -> 1=yes, 0=no */
#define DO_INTERP            1   /* preform interpolation on bands 23 and 24 -> 1=yes, 0=no */
#define NB_LP_BANDS_CODING   3
#define ENERGY_CORR          1   /* correct log energy by using the HP energy */
#define PERC_CODED           0.7

#define NE16k_FRAMES_THRESH  100
#define LAMBDA_NSE16k        0.99

typedef struct QMF_FIR QMF_FIR;
typedef struct HP_FIL_IIR HP_FIL_IIR;
typedef struct DataFor16kProc_F DataFor16kProc_F ;

/* public functions */
extern QMF_FIR *        QMF_FIR_Init_F           (void);
extern void             hq_free_F                (QMF_FIR *fir_ptr);
extern HP_FIL_IIR *     HPfilter_init_F          (void);
extern void             DoHPfiltering_F          (float *inData, float *outData, HP_FIL_IIR *pHPfil, int inDataLength);
extern void             Do16kProcDelete_F        (DataFor16kProc_F * pData16k);
extern void             Do16kProcInit_F          (FEParamsX_F * This);
extern struct DataFor16kProc_F * Do16kProcAlloc_F         (void);
extern void             Do16kProcessing_F      (float *inData,DataFor16kProc_F * pData16k, int inDataLength);
extern void             GetBandsForCoding16k_F   (float *inData, float *bandsForCoding16k, int inDataSize);
extern void             CodeBands16k_F           (float *fb16k, float *lpBands, float *codeForBands16k);
extern void             GetBandsForDecoding16k_F (float *inData, float *bandsForCoding16k, int inDataSize);
extern void             DecodeBands16k_F         (float *fb16k, float *lpBands, float *codeForBands16k, float *codeWeights);
extern void             DoSpecSub16k_F           (float *inFB16k,DataFor16kProc_F * pData16k, long vadCounter16k, int inFB16kSize);
extern void             MergeSSandCoded_F        (float * FloatBuffer, float * hp16kBands, int numChannelsWI7, X_INT16 * hp16kBandsSize_in, 
						struct DataFor16kProc_F * pData16k);
extern float            CorrectEnergy_F          (float LogEnergy, int hp16kBandsSize, float * FloatBuffer, int numChannelsWI7, float preem);

/* public get functions */
extern int     Get16k_bufData16kSize_F          (DataFor16kProc_F * pData16k);
extern int     Get16k_hpBandsSize_F             (DataFor16kProc_F * pData16k);
extern float   Get16k_percCoded_F               (DataFor16kProc_F * pData16k);
extern float * Get16k_p_hpBands_F               (DataFor16kProc_F * pData16k);
extern float * Get16k_p_bufferCodeForBands16k_F (DataFor16kProc_F * pData16k);
extern float * Get16k_p_CodeForBands16_F        (DataFor16kProc_F * pData16k);
extern float * Get16k_p_bufferCodeWeights_F     (DataFor16kProc_F * pData16k);
extern float * Get16k_p_codeWeights_F           (DataFor16kProc_F * pData16k);
extern float * Get16k_p_bufferData16k_F         (DataFor16kProc_F * pData16k);
extern float * Get16k_p_BandsForCoding16k_F     (DataFor16kProc_F * pData16k);
extern MelFB_Window * Get16k_p_FirstWindow16k_F (DataFor16kProc_F * pData16k);
extern float * Get16k_p_CodeForBands16k_F		(DataFor16kProc_F *pData16k);

extern float   Get16k_dataHP_F               (DataFor16kProc_F * pData16k,int i);


#endif
