/*=======================================================================================================*/
/*      Distributed Speech Recognition                                                                   */
/*      ETSI ES 202 050 Advanced Front-End Feature Extraction Algorithm & Compression Algorithm &        */
/*      ETSI ES 202 212 Extended Advanced Front-End Feature Extraction Algorithm & Compression Algorithm */  
/*      Bit exact C-language software implementation using ETSI basic operations library                 */
/*      3GPP TS 26.243 v 6.1.0 November 2004                                                                  */
/*      ANSI C code for the Fixed-Point Distributed Speech Recognition Extended Advanced Front-end       */
/*=======================================================================================================*/
/*
***********************************************************************
*
*   FILE NAME:
*
*      rvc_pitch_api_B.h
*
*   PURPOSE:
*
*      Header file for API functions of the pitch extractor
*
**********************************************************************
*/


#include "ParmType.h"

#ifndef RVC_PITCH_API
#define RVC_PITCH_API

#ifdef __cplusplus
extern "C" {
#endif


/*
 * Enumeration of return arguments
 */

typedef enum {

  /// Function terminated successfully (zero value)
  RVC_OK = 0,

  /// Function terminated with a general error. The type of error is not specified.
  RVC_ERR = -1,

  /// Function failed allocating memory for a new array (malloc failed)
  RVC_ERR_NOT_ENOUGH_MEMORY =  -2,

  /** One of the function input or output argument is illegal. 
  This for example can be a NULL address pointer, a value out 
  of bounds, etc. */
  RVC_ERR_ILLEGAL_ARGUMENT =   -3,

  /** The function failed writing to an output file or reading 
  from an input file. */
  RVC_ERR_IO_FAILED = -4,

  /** Failed parsing a file format header. The file
  may be corrupted or it is not a legal file */
  RVC_ERR_BAD_FILE_FORMAT = -5,

  /** Function failed because one of the encoder
  or decoder initialization methods was not called first */
  RVC_ERR_NOT_INITIALIZED = -6,

  /** Illegal usage of this function. One can not call this
  function at this time */
  RVC_ERR_ILLEGAL_USAGE = -7,

  /// Not enough samples to process
  RVC_ERR_NOT_ENOUGH_SAMPLES =  -8,

  /** This function is not yet implemented. Currently this
  may be returned by some of the fixed point functions, which were
  yet written */
  RVC_ERR_NOT_IMPLEMENTED = -9,

  /** This function failed opening the file specified for reading or 
  writing */
  RVC_ERR_FAIL_OPEN_FILE = -10

} RvcResult;


X_INT16 RVC_ConstructPitchRom_be(const X_INT16 iSampleRate,
		                 const Shortword swPreEmphFactor,
		                 void **pRvcPitchRom);

X_INT16 RVC_ConstructPitchMeter_be(const void *RvcPitchRom, 
			           void **pRvcPitchMeter);

X_INT16 RVC_MeasurePitch_be(const void *pvRvcPitchRom,
                            void *pvRvcPitchMeter,
                            const Longword lwSpecAverage,
                            Shortword *pswProcSpeechForCorr,
                            const Shortword *pswStft,
                            const X_INT16 iStftShift,
                            const Longword *plwPowerSpectrum,
                            const X_INT16 iPowerSpectrumShift,
                            const NormSw nsFrameEnergy, 
                            const X_INT16 iVad, // boolean
                            const X_INT16 iLowBandNoiseFlag, // boolean
                            Longword *plwOutPitchPeriod);

X_INT16 RVC_ResetPitchMeter_be(void *RvcPitchMeter);

X_INT16 RVC_DestructPitchRom_be(void *RvcPitchRom);

X_INT16 RVC_DestructPitchMeter_be(void *RvcPitchMeter);
		

#ifdef __cplusplus
}
#endif

#endif //RVC_PITCH_API

