#=======================================================================================================
#       Distributed Speech Recognition                                                                  
#       ETSI ES 202 050 Advanced Front-End Feature Extraction Algorithm & Compression Algorithm &       
#       ETSI ES 202 212 Extended Advanced Front-End Feature Extraction Algorithm & Compression Algorithm  
#       Bit exact C-language software implementation using ETSI basic operations library                
#       3GPP TS 26.243 v 6.1.0 November 2004                                                                 
#       ANSI C code for the Fixed-Point Distributed Speech Recognition Extended Advanced Front-end
#=======================================================================================================
#
# Makefile for compiling all software modules 
#
.SUFFIXES: .o .a .c .h

#
# CONFIG specifies OS dependent commands          
#
CONFIG=config.cfg

include $(CONFIG)

#
# Specify dir path for source, object, lib, etc...

#
PROG2_SOURCE=../src
INTER_SOURCE=../interface
PROG2_INCLUDE=$(PROG2_SOURCE)
PROG2_OBJ=./obj
PROG2_LIB=./lib
PROG2_EXE=./bin

all : \
    ExtAdvFrontEnd		\
    AdvFrontEnd			\
	coder		\
	decoder		\
	ExtCoder		\
	ExtDecoder		\
	Reconstruct		\
	PitchTrack		\
	error_handle


###############################################################################
# Specify include paths
#

CUR_INCLUDE_XAFE=  \
             -I../../basicops -I../../cleanfinalFix/NoiseSup \
             -I../../cleanfinalFix/PostProc \
             -I../../cleanfinalFix/WaveProc -I../../cleanfinalFix/CompCeps  \
             -I../../cleanfinalFix/VAD -I../../cleanfinalFix/16KHzProc \
             -I../../cleanfinalFix -I../../cleanfinalFix/Extension

CUR_INCLUDE_CODER=  \
             -I../../vq -I../../basicops

CUR_INCLUDE_RECON=  \
             -I../../Reconstruction


#
# For basicops debug add -DXB_DEBUG
#
# For 16Khz version add -DF_16KHZ

CUR_CC= $(CC_CMD_OPT) $(CUR_INCLUDE_XAFE)  -DF_16KHZ

CUR_CC_XAFE= $(CC_CMD_OPT) $(CUR_INCLUDE_XAFE) -DF_16KHZ -DXAFE

CUR_CC_CODER= $(CC_CMD_OPT) $(CUR_INCLUDE_CODER)

CUR_CC_CODER_XAFE= $(CC_CMD_OPT) $(CUR_INCLUDE_CODER) -DXAFE

CUR_CC_RECON= $(CC_CMD_OPT) $(CUR_INCLUDE_RECON)


####################### default rules #############################


clean :
	$(DELETE) $(PROG2_OBJ)/*.o $(PROG2_EXE)/*

ExtAdvFrontEnd: $(PROG2_EXE)/ExtAdvFrontEnd


LKDP_XAFE =		$(PROG2_OBJ)/ExtAdvFrontEnd_B.o  \
			$(PROG2_OBJ)/Xbascount.o 	 \
			$(PROG2_OBJ)/Xbasicops.o 	 \
			$(PROG2_OBJ)/basicop2.o				\
			$(PROG2_OBJ)/count.o				\
			$(PROG2_OBJ)/BufferIn32.o	 \
			$(PROG2_OBJ)/ParmInterface_B.o   \
			$(PROG2_OBJ)/PostProc_B.o        \
			$(PROG2_OBJ)/WaveProc_B.o        \
			$(PROG2_OBJ)/CompCeps_B.o    \
			$(PROG2_OBJ)/fileio.o            \
			$(PROG2_OBJ)/ExtNoiseSup_B.o    \
			$(PROG2_OBJ)/ff4nrFix32_B.o  	 \
			$(PROG2_OBJ)/ff4nrFix16_B.o  	 \
			$(PROG2_OBJ)/16kHzProcessing_B.o \
			$(PROG2_OBJ)/VAD_F.o				   \
			$(PROG2_OBJ)/MathFunc.o				\
			$(PROG2_OBJ)/classifyFrame_B.o	 \
			$(PROG2_OBJ)/computePCorr_B.o	 \
			$(PROG2_OBJ)/dsrAfeVad_B.o	 \
			$(PROG2_OBJ)/preProc_B.o	 \
			$(PROG2_OBJ)/qsort_B.o		 \
			$(PROG2_OBJ)/fix_mathlib.o	 \
			$(PROG2_OBJ)/rvc_pitch_B.o	 \
			$(PROG2_OBJ)/rvc_pitch_init_B.o



$(PROG2_EXE)/ExtAdvFrontEnd :  $(LKDP_XAFE) 		
	$(LINK_CMD_SO)  -L$(PROG2_LIB) $(LKDP_XAFE) -lm  $(LK_OUT) $@


AdvFrontEnd: $(PROG2_EXE)/AdvFrontEnd

LKDP_NOXAFE =		$(PROG2_OBJ)/ExtAdvFrontEnd_B_NOXAFE.o  \
			$(PROG2_OBJ)/Xbascount.o 	 \
			$(PROG2_OBJ)/Xbasicops.o 	 \
			$(PROG2_OBJ)/basicop2.o				\
			$(PROG2_OBJ)/count.o				\
			$(PROG2_OBJ)/BufferIn32.o	 \
			$(PROG2_OBJ)/ParmInterface_B_NOXAFE.o   \
			$(PROG2_OBJ)/PostProc_B.o        \
			$(PROG2_OBJ)/WaveProc_B.o        \
			$(PROG2_OBJ)/CompCeps_B.o    \
			$(PROG2_OBJ)/fileio.o            \
			$(PROG2_OBJ)/ExtNoiseSup_B_NOXAFE.o    \
			$(PROG2_OBJ)/ff4nrFix32_B.o  	 \
			$(PROG2_OBJ)/ff4nrFix16_B.o  	 \
			$(PROG2_OBJ)/16kHzProcessing_B.o \
			$(PROG2_OBJ)/VAD_F.o				   \
			$(PROG2_OBJ)/MathFunc.o			


$(PROG2_EXE)/AdvFrontEnd :  $(LKDP_NOXAFE) 		
	$(LINK_CMD_SO)  -L$(PROG2_LIB) $(LKDP_NOXAFE) -lm  $(LK_OUT) $@


coder: $(PROG2_EXE)/coder

LKDP_coder_NOXAFE = 	$(PROG2_OBJ)/coder_NOXAFE.o	\
			$(PROG2_OBJ)/golay_NOXAFE.o		\
			$(PROG2_OBJ)/basicop2.o		\
			$(PROG2_OBJ)/count.o

$(PROG2_EXE)/coder :  $(LKDP_coder_NOXAFE)
	$(LINK_CMD_SO)  -L$(PROG2_LIB) $(LKDP_coder_NOXAFE) -lm $(LK_OUT) $@


ExtCoder: $(PROG2_EXE)/ExtCoder

LKDP_coder = $(PROG2_OBJ)/coder.o	\
			$(PROG2_OBJ)/golay.o		\
			$(PROG2_OBJ)/basicop2.o		\
			$(PROG2_OBJ)/count.o

$(PROG2_EXE)/ExtCoder :  $(LKDP_coder)
	$(LINK_CMD_SO)  -L$(PROG2_LIB) $(LKDP_coder) -lm $(LK_OUT) $@




decoder: $(PROG2_EXE)/decoder

LKDP_decoder_NOXAFE = 	$(PROG2_OBJ)/decoder_NOXAFE.o	\
			$(PROG2_OBJ)/golay_NOXAFE.o		\
			$(PROG2_OBJ)/basicop2.o		\
			$(PROG2_OBJ)/count.o

$(PROG2_EXE)/decoder :  $(LKDP_decoder_NOXAFE)
	$(LINK_CMD_SO)  -L$(PROG2_LIB) $(LKDP_decoder_NOXAFE) -lm $(LK_OUT) $@

ExtDecoder: $(PROG2_EXE)/ExtDecoder

LKDP_decoder = 	$(PROG2_OBJ)/decoder.o	\
			$(PROG2_OBJ)/golay.o		\
			$(PROG2_OBJ)/basicop2.o		\
			$(PROG2_OBJ)/count.o

$(PROG2_EXE)/ExtDecoder :  $(LKDP_decoder)
	$(LINK_CMD_SO)  -L$(PROG2_LIB) $(LKDP_decoder) -lm $(LK_OUT) $@



Reconstruct: $(PROG2_EXE)/Reconstruct

LKDP_Reconstruct =	$(PROG2_OBJ)/main.o		\
			$(PROG2_OBJ)/init.o		\
			$(PROG2_OBJ)/fft.o		\
			$(PROG2_OBJ)/utils.o		\
			$(PROG2_OBJ)/reconstruct.o	\
			$(PROG2_OBJ)/phase.o		\
			$(PROG2_OBJ)/bfun_mag.o		\
			$(PROG2_OBJ)/interp_mag.o	\
			$(PROG2_OBJ)/APmodel.o

$(PROG2_EXE)/Reconstruct :  $(LKDP_Reconstruct)
	$(LINK_CMD_SO)  -L$(PROG2_LIB) $(LKDP_Reconstruct) -lm $(LK_OUT) $@


PitchTrack: $(PROG2_EXE)/PitchTrack

$(PROG2_EXE)/PitchTrack :  $(PROG2_OBJ)/PitchTrack.o
	$(LINK_CMD_SO)  $(PROG2_OBJ)/PitchTrack.o -lm $(LK_OUT) $@

error_handle: $(PROG2_EXE)/error_handle

$(PROG2_EXE)/error_handle : $(PROG2_OBJ)/error_handle.o
	$(LINK_CMD_SO) $(PROG2_OBJ)/error_handle.o $(LK_OUT) $@



############################## General Modules ###########################



$(PROG2_OBJ)/BufferIn32.o : \
	../../cleanfinalFix/BufferIn32.c \
	../../cleanfinalFix/BufferIn32.h \
	../../cleanfinalFix/ParmType.h \
	../../cleanfinalFix/x_default.h 
		$(CUR_CC) $(CC_OUT) $@  ../../cleanfinalFix/$(@F:.o=.c) 

$(PROG2_OBJ)/CompCeps_B.o : \
	../../cleanfinalFix/CompCeps/CompCeps_B.c \
	../../basicops/Xbascount.h \
	../../basicops/Xbasicops.h \
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/BufferIn32.h \
	../../cleanfinalFix/ParmInterface_B.h \
	../../cleanfinalFix/ParmType.h \
	../../cleanfinalFix/x_default.h \
	../../cleanfinalFix/16KHzProc/16kHzProcExports_B.h \
     ../../cleanfinalFix/16KHzProc/16kHzProcExports_F.h	\
	../../cleanfinalFix/CompCeps/CompCepsExports_B.h \
	../../cleanfinalFix/CompCeps/ff4nrFix16_B.h
		$(CUR_CC) $(CC_OUT) $@  ../../cleanfinalFix/CompCeps/$(@F:.o=.c) 

$(PROG2_OBJ)/ff4nrFix32_B.o : \
	../../cleanfinalFix/CompCeps/ff4nrFix32_B.c \
	../../basicops/Xbascount.h \
	../../basicops/Xbasicops.h \
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/x_default.h \
	../../cleanfinalFix/CompCeps/ff4nrFix32_B.h 
		$(CUR_CC) $(CC_OUT) $@  ../../cleanfinalFix/CompCeps/$(@F:.o=.c) 

$(PROG2_OBJ)/MathFunc.o : \
	../../cleanfinalFix/CompCeps/MathFunc.c \
	../../basicops/Xbascount.h \
	../../basicops/Xbasicops.h \
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/x_default.h \
	../../cleanfinalFix/CompCeps/MathFunc.h 
		$(CUR_CC) $(CC_OUT) $@  ../../cleanfinalFix/CompCeps/$(@F:.o=.c) 

$(PROG2_OBJ)/ff4nrFix16_B.o : \
	../../cleanfinalFix/CompCeps/ff4nrFix16_B.c \
	../../basicops/Xbascount.h \
	../../basicops/Xbasicops.h \
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/x_default.h \
	../../cleanfinalFix/CompCeps/ff4nrFix16_B.h 
		$(CUR_CC) $(CC_OUT) $@  ../../cleanfinalFix/CompCeps/$(@F:.o=.c) 

$(PROG2_OBJ)/ff4nrFix.o : \
	../../cleanfinalFix/CompCeps/ff4nrFix.c \
	../../cleanfinalFix/x_default.h \
	../../cleanfinalFix/CompCeps/ff4nrFix.h 
		$(CUR_CC) $(CC_OUT) $@  ../../cleanfinalFix/CompCeps/$(@F:.o=.c) 

$(PROG2_OBJ)/ff4nrFix32.o : \
	../../cleanfinalFix/CompCeps/ff4nrFix32.c \
	../../cleanfinalFix/x_default.h \
	../../cleanfinalFix/CompCeps/ff4nrFix32.h 
		$(CUR_CC) $(CC_OUT) $@  ../../cleanfinalFix/CompCeps/$(@F:.o=.c) 

$(PROG2_OBJ)/fileio.o : \
	../../cleanfinalFix/fileio.c \
	../../cleanfinalFix/fileio.h 
		$(CUR_CC) $(CC_OUT) $@  ../../cleanfinalFix/$(@F:.o=.c) 

$(PROG2_OBJ)/NoiseSup_B.o : \
	../../cleanfinalFix/NoiseSup/NoiseSup_B.c \
	../../basicops/Xbascount.h \
	../../basicops/Xbasicops.h \
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/BufferIn32.h \
	../../cleanfinalFix/ParmInterface_B.h \
	../../cleanfinalFix/ParmType.h \
	../../cleanfinalFix/x_default.h \
     ../../cleanfinalFix/16KHzProc/16kHzProcExports_B.h	\
     ../../cleanfinalFix/16KHzProc/16kHzProcExports_F.h	\
	../../cleanfinalFix/CompCeps/ff4nrFix32_B.h \
	../../cleanfinalFix/NoiseSup/NoiseSup_F.h \
	../../cleanfinalFix/NoiseSup/NoiseSupExports_B.h 
		$(CUR_CC) $(CC_OUT) $@  ../../cleanfinalFix/NoiseSup/$(@F:.o=.c) 

$(PROG2_OBJ)/PostProc_B.o : \
	../../cleanfinalFix/PostProc/PostProc_B.c \
	../../basicops/Xbascount.h \
	../../basicops/Xbasicops.h \
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/BufferIn32.h \
	../../cleanfinalFix/ParmInterface_B.h \
	../../cleanfinalFix/ParmType.h \
	../../cleanfinalFix/x_default.h \
	../../cleanfinalFix/PostProc/PostProcExports_B.h 
		$(CUR_CC) $(CC_OUT) $@  ../../cleanfinalFix/PostProc/$(@F:.o=.c) 

$(PROG2_OBJ)/VAD_F.o : \
	../../cleanfinalFix/VAD/VAD_F.c \
	../../cleanfinalFix/BufferIn32.h \
	../../cleanfinalFix/ParmInterface_B.h \
	../../cleanfinalFix/ParmType.h \
	../../cleanfinalFix/x_default.h \
	../../cleanfinalFix/VAD/VADExports_F.h 
		$(CUR_CC) $(CC_OUT) $@  ../../cleanfinalFix/VAD/$(@F:.o=.c) 

$(PROG2_OBJ)/WaveProc_B.o : \
	../../cleanfinalFix/WaveProc/WaveProc_B.c \
	../../basicops/Xbascount.h \
	../../basicops/Xbasicops.h \
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/BufferIn32.h \
	../../cleanfinalFix/ParmInterface_B.h \
	../../cleanfinalFix/ParmType.h \
	../../cleanfinalFix/x_default.h \
	../../cleanfinalFix/WaveProc/WaveProcExports_B.h 
		$(CUR_CC) $(CC_OUT) $@  ../../cleanfinalFix/WaveProc/$(@F:.o=.c) 

$(PROG2_OBJ)/Xbascount.o : \
	../../basicops/Xbascount.c  \
	../../basicops/Xbascount.h \
	../../basicops/Xbascountpriv.h \
	../../cleanfinalFix/x_default.h 
		$(CUR_CC) $(CC_OUT) $@  ../../basicops/$(@F:.o=.c) 

$(PROG2_OBJ)/Xbasicops.o : \
	../../basicops/Xbasicops.c  \
	../../basicops/Xbasicops.h  \
	../../basicops/Xbascount.h \
	../../basicops/Xbascountpriv.h \
	../../cleanfinalFix/x_default.h 
		$(CUR_CC) $(CC_OUT) $@  ../../basicops/$(@F:.o=.c) 


#
# Only for 16_khz
#

$(PROG2_OBJ)/16kHzProcessing_B.o : \
	../../cleanfinalFix/16KHzProc/16kHzProcessing_B.c  \
	../../basicops/Xbascount.h\
	../../basicops/Xbasicops.h\
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/BufferIn32.h \
	../../cleanfinalFix/ParmInterface_B.h \
	../../cleanfinalFix/ParmType.h \
	../../cleanfinalFix/x_default.h \
	../../cleanfinalFix/16KHzProc/16kHzProcExports_B.h \
	../../cleanfinalFix/16KHzProc/16kHzProcExports_F.h \
	../../cleanfinalFix/NoiseSup/NoiseSupExports_F.h
	$(CUR_CC) $(CC_OUT) $@  ../../cleanfinalFix/16KHzProc/$(@F:.o=.c)


############################ ExtAdvFrontEnd modules ##########################


$(PROG2_OBJ)/ExtAdvFrontEnd_B.o : \
	../../cleanfinalFix/ExtAdvFrontEnd_B.c \
	../../basicops/Xbascount.h \
	../../basicops/Xbasicops.h \
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/BufferIn32.h \
	../../cleanfinalFix/fileio.h \
	../../cleanfinalFix/ParmInterface_B.h \
	../../cleanfinalFix/ParmType.h \
	../../cleanfinalFix/x_default.h \
	../../cleanfinalFix/Extension/rvc_pitch_api_B.h
	$(CUR_CC_XAFE) $(CC_OUT) $@  ../../cleanfinalFix/$(@F:.o=.c) 

#
# WARNING XAFE not defined !!
#
$(PROG2_OBJ)/ExtAdvFrontEnd_B_NOXAFE.o : \
	../../cleanfinalFix/ExtAdvFrontEnd_B.c \
	../../basicops/Xbascount.h \
	../../basicops/Xbasicops.h \
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/BufferIn32.h \
	../../cleanfinalFix/fileio.h \
	../../cleanfinalFix/ParmInterface_B.h \
	../../cleanfinalFix/ParmType.h \
	../../cleanfinalFix/x_default.h \
	../../cleanfinalFix/Extension/rvc_pitch_api_B.h
	$(CUR_CC) $(CC_OUT) $@  ../../cleanfinalFix/ExtAdvFrontEnd_B.c 



$(PROG2_OBJ)/ParmInterface_B.o : \
	../../cleanfinalFix/ParmInterface_B.c  \
	../../cleanfinalFix/BufferIn32.h \
	../../cleanfinalFix/ParmType.h \
	../../cleanfinalFix/x_default.h \
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/16KHzProc/16kHzProcExports_B.h	\
	../../cleanfinalFix/CompCeps/CompCepsExports_B.h \
	../../cleanfinalFix/NoiseSup/NoiseSupExports_B.h \
	../../cleanfinalFix/ParmInterface_B.h \
	../../cleanfinalFix/PostProc/PostProcExports_B.h \
	../../cleanfinalFix/VAD/VADExports_F.h \
	../../cleanfinalFix/WaveProc/WaveProcExports_B.h 
		$(CUR_CC_XAFE) $(CC_OUT) $@  ../../cleanfinalFix/$(@F:.o=.c) 

#
# WARNING XAFE not defined !!
#
$(PROG2_OBJ)/ParmInterface_B_NOXAFE.o : \
	../../cleanfinalFix/ParmInterface_B.c  \
	../../cleanfinalFix/BufferIn32.h \
	../../cleanfinalFix/ParmType.h \
	../../cleanfinalFix/x_default.h \
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/16KHzProc/16kHzProcExports_B.h	\
	../../cleanfinalFix/CompCeps/CompCepsExports_B.h \
	../../cleanfinalFix/NoiseSup/NoiseSupExports_B.h \
	../../cleanfinalFix/ParmInterface_B.h \
	../../cleanfinalFix/PostProc/PostProcExports_B.h \
	../../cleanfinalFix/VAD/VADExports_F.h \
	../../cleanfinalFix/WaveProc/WaveProcExports_B.h 
		$(CUR_CC) $(CC_OUT) $@  ../../cleanfinalFix/ParmInterface_B.c



$(PROG2_OBJ)/ExtNoiseSup_B.o : \
	../../cleanfinalFix/NoiseSup/ExtNoiseSup_B.c \
	../../basicops/Xbascount.h \
	../../basicops/Xbasicops.h \
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/BufferIn32.h \
	../../cleanfinalFix/ParmInterface_B.h \
	../../cleanfinalFix/ParmType.h \
	../../cleanfinalFix/x_default.h \
        ../../cleanfinalFix/16KHzProc/16kHzProcExports_B.h	\
        ../../cleanfinalFix/16KHzProc/16kHzProcExports_F.h	\
	../../cleanfinalFix/CompCeps/ff4nrFix32_B.h \
	../../cleanfinalFix/CompCeps/CompCepsExports_B.h \
	../../cleanfinalFix/NoiseSup/NoiseSup_F.h \
	../../cleanfinalFix/NoiseSup/NoiseSupExports_B.h \
        ../../cleanfinalFix/Extension/rvc_pitch_api_B.h  \
        ../../cleanfinalFix/Extension/classifyFrame_B.h  \
        ../../cleanfinalFix/Extension/dsrAfeVad_B.h  \
        ../../cleanfinalFix/Extension/preProc_B.h
	$(CUR_CC_XAFE) $(CC_OUT) $@  ../../cleanfinalFix/NoiseSup/$(@F:.o=.c) 

#
# WARNING XAFE not defined !!
#
$(PROG2_OBJ)/ExtNoiseSup_B_NOXAFE.o : \
	../../cleanfinalFix/NoiseSup/ExtNoiseSup_B.c \
	../../basicops/Xbascount.h \
	../../basicops/Xbasicops.h \
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/BufferIn32.h \
	../../cleanfinalFix/ParmInterface_B.h \
	../../cleanfinalFix/ParmType.h \
	../../cleanfinalFix/x_default.h \
        ../../cleanfinalFix/16KHzProc/16kHzProcExports_B.h	\
        ../../cleanfinalFix/16KHzProc/16kHzProcExports_F.h	\
	../../cleanfinalFix/CompCeps/ff4nrFix32_B.h \
	../../cleanfinalFix/CompCeps/CompCepsExports_B.h \
	../../cleanfinalFix/NoiseSup/NoiseSup_F.h \
	../../cleanfinalFix/NoiseSup/NoiseSupExports_B.h \
        ../../cleanfinalFix/Extension/rvc_pitch_api_B.h  \
        ../../cleanfinalFix/Extension/classifyFrame_B.h  \
        ../../cleanfinalFix/Extension/dsrAfeVad_B.h  \
        ../../cleanfinalFix/Extension/preProc_B.h
	$(CUR_CC) $(CC_OUT) $@  ../../cleanfinalFix/NoiseSup/ExtNoiseSup_B.c 



$(PROG2_OBJ)/rvc_pitch_B.o : \
	../../cleanfinalFix/Extension/rvc_pitch_B.c \
	../../basicops/Xbascount.h \
	../../basicops/Xbasicops.h \
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/ParmType.h \
	../../cleanfinalFix/x_default.h \
        ../../cleanfinalFix/Extension/rvc_pitch_B.h  \
        ../../cleanfinalFix/Extension/rvc_pitch_api_B.h  \
        ../../cleanfinalFix/Extension/computePCorr_B.h	 \
        ../../cleanfinalFix/Extension/qsort_B.h		 \
        ../../cleanfinalFix/Extension/fix_mathlib.h
	$(CUR_CC_XAFE) $(CC_OUT) $@  ../../cleanfinalFix/Extension/$(@F:.o=.c) 





$(PROG2_OBJ)/rvc_pitch_init_B.o : \
	../../cleanfinalFix/Extension/rvc_pitch_init_B.c \
	../../basicops/Xbascount.h \
	../../basicops/Xbasicops.h \
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/ParmType.h \
	../../cleanfinalFix/x_default.h \
        ../../cleanfinalFix/Extension/rvc_pitch_B.h  \
        ../../cleanfinalFix/Extension/rvc_pitch_api_B.h
	$(CUR_CC_XAFE) $(CC_OUT) $@  ../../cleanfinalFix/Extension/$(@F:.o=.c) 





$(PROG2_OBJ)/dsrAfeVad_B.o : \
	../../cleanfinalFix/Extension/dsrAfeVad_B.c \
	../../basicops/Xbascount.h \
	../../basicops/Xbasicops.h \
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/ParmType.h \
	../../cleanfinalFix/x_default.h \
	../../cleanfinalFix/Extension/dsrAfeVad_B.h
	$(CUR_CC_XAFE) $(CC_OUT) $@  ../../cleanfinalFix/Extension/$(@F:.o=.c) 




$(PROG2_OBJ)/classifyFrame_B.o : \
	../../cleanfinalFix/Extension/classifyFrame_B.c \
	../../basicops/Xbascount.h \
	../../basicops/Xbasicops.h \
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/ParmType.h \
	../../cleanfinalFix/x_default.h \
	../../cleanfinalFix/Extension/classifyFrame_B.h
	$(CUR_CC_XAFE) $(CC_OUT) $@  ../../cleanfinalFix/Extension/$(@F:.o=.c) 




$(PROG2_OBJ)/computePCorr_B.o : \
	../../cleanfinalFix/Extension/computePCorr_B.c \
	../../basicops/Xbascount.h \
	../../basicops/Xbasicops.h \
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/ParmType.h \
	../../cleanfinalFix/x_default.h \
	../../cleanfinalFix/Extension/fix_mathlib.h \
	../../cleanfinalFix/Extension/computePCorr_B.h
	$(CUR_CC_XAFE) $(CC_OUT) $@  ../../cleanfinalFix/Extension/$(@F:.o=.c) 




$(PROG2_OBJ)/preProc_B.o : \
	../../cleanfinalFix/Extension/preProc_B.c \
	../../basicops/Xbascount.h \
	../../basicops/Xbasicops.h \
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/ParmType.h \
	../../cleanfinalFix/x_default.h \
	../../cleanfinalFix/Extension/preProc_B.h
	$(CUR_CC_XAFE) $(CC_OUT) $@  ../../cleanfinalFix/Extension/$(@F:.o=.c) 




$(PROG2_OBJ)/qsort_B.o : \
	../../cleanfinalFix/Extension/qsort_B.c \
	../../basicops/Xbascount.h \
	../../basicops/Xbasicops.h \
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/ParmType.h \
	../../cleanfinalFix/x_default.h \
	../../cleanfinalFix/Extension/qsort_B.h
	$(CUR_CC_XAFE) $(CC_OUT) $@  ../../cleanfinalFix/Extension/$(@F:.o=.c) 




$(PROG2_OBJ)/fix_mathlib.o : \
	../../cleanfinalFix/Extension/fix_mathlib.c \
	../../basicops/Xbascount.h \
	../../basicops/Xbasicops.h \
	../../basicops/basic_op.h	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../cleanfinalFix/ParmType.h \
	../../cleanfinalFix/x_default.h \
	../../cleanfinalFix/Extension/fix_mathlib.h
	$(CUR_CC_XAFE) $(CC_OUT) $@  ../../cleanfinalFix/Extension/$(@F:.o=.c) 




########################### Coder & Decoder modules ########################


$(PROG2_OBJ)/coder.o : \
	../../vq/coder.c	\
	../../vq/coder.h	\
	../../vq/common.h	\
	../../vq/golay.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../basicops/basic_op.h	\
	../../vq/quantiser_8kHz_BEC.tab		\
	../../vq/quantiser_16kHz_BEC.tab
	$(CUR_CC_CODER_XAFE) $(CC_OUT) $@  ../../vq/$(@F:.o=.c)

$(PROG2_OBJ)/coder_NOXAFE.o : \
	../../vq/coder.c	\
	../../vq/coder.h	\
	../../vq/common.h	\
	../../vq/golay.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../basicops/basic_op.h	\
	../../vq/quantiser_8kHz_BEC.tab		\
	../../vq/quantiser_16kHz_BEC.tab
	$(CUR_CC_CODER) $(CC_OUT) $@  ../../vq/coder.c




$(PROG2_OBJ)/decoder.o : \
	../../vq/decoder.c	\
	../../vq/decoder.h	\
	../../vq/common.h	\
	../../vq/golay.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../vq/quantiser_8kHz_BEC.tab		\
	../../vq/quantiser_16kHz_BEC.tab
	$(CUR_CC_CODER_XAFE) $(CC_OUT) $@  ../../vq/$(@F:.o=.c)

$(PROG2_OBJ)/decoder_NOXAFE.o : \
	../../vq/decoder.c	\
	../../vq/decoder.h	\
	../../vq/common.h	\
	../../vq/golay.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../vq/quantiser_8kHz_BEC.tab		\
	../../vq/quantiser_16kHz_BEC.tab
	$(CUR_CC_CODER) $(CC_OUT) $@  ../../vq/decoder.c



$(PROG2_OBJ)/golay.o : \
	../../vq/golay.c	\
	../../vq/golay.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../basicops/basic_op.h
	$(CUR_CC_CODER_XAFE) $(CC_OUT) $@  ../../vq/$(@F:.o=.c)

$(PROG2_OBJ)/golay_NOXAFE.o : \
	../../vq/golay.c	\
	../../vq/golay.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h	\
	../../basicops/basic_op.h
	$(CUR_CC_CODER) $(CC_OUT) $@  ../../vq/golay.c





$(PROG2_OBJ)/basicop2.o : \
	../../basicops/basicop2.c	\
	../../basicops/basic_op.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h
	$(CUR_CC_CODER) $(CC_OUT) $@  ../../basicops/$(@F:.o=.c)





$(PROG2_OBJ)/count.o : \
	../../basicops/count.c	\
	../../basicops/count.h	\
	../../basicops/typedef.h	\
	../../basicops/typedefs.h
	$(CUR_CC_CODER) $(CC_OUT) $@  ../../basicops/$(@F:.o=.c)

####################### Reconstruction modules ########################



$(PROG2_OBJ)/main.o : \
	../../Reconstruction/main.c		\
	../../Reconstruction/xdsr_reconst_api.h
	$(CUR_CC_RECON) $(CC_OUT) $@  ../../Reconstruction/$(@F:.o=.c)




$(PROG2_OBJ)/init.o :	\
	../../Reconstruction/init.c		\
	../../Reconstruction/init_tab.h		\
	../../Reconstruction/common.h
	$(CUR_CC_RECON) $(CC_OUT) $@  ../../Reconstruction/$(@F:.o=.c)




$(PROG2_OBJ)/fft.o :	\
	../../Reconstruction/fft.c
	$(CUR_CC_RECON) $(CC_OUT) $@  ../../Reconstruction/$(@F:.o=.c)




$(PROG2_OBJ)/utils.o :	\
	../../Reconstruction/utils.c		\
	../../Reconstruction/common.h
	$(CUR_CC_RECON) $(CC_OUT) $@  ../../Reconstruction/$(@F:.o=.c)




$(PROG2_OBJ)/reconstruct.o :	\
	../../Reconstruction/reconstruct.c		\
	../../Reconstruction/common.h
	$(CUR_CC_RECON) $(CC_OUT) $@  ../../Reconstruction/$(@F:.o=.c)




$(PROG2_OBJ)/phase.o :	\
	../../Reconstruction/phase.c		\
	../../Reconstruction/common.h
	$(CUR_CC_RECON) $(CC_OUT) $@  ../../Reconstruction/$(@F:.o=.c)





$(PROG2_OBJ)/bfun_mag.o :	\
	../../Reconstruction/bfun_mag.c		\
	../../Reconstruction/common.h
	$(CUR_CC_RECON) $(CC_OUT) $@  ../../Reconstruction/$(@F:.o=.c)





$(PROG2_OBJ)/interp_mag.o :	\
	../../Reconstruction/interp_mag.c		\
	../../Reconstruction/common.h
	$(CUR_CC_RECON) $(CC_OUT) $@  ../../Reconstruction/$(@F:.o=.c)

$(PROG2_OBJ)/APmodel.o :	\
	../../Reconstruction/APmodel.c		\
	../../Reconstruction/common.h
	$(CUR_CC_RECON) $(CC_OUT) $@  ../../Reconstruction/$(@F:.o=.c)

$(PROG2_OBJ)/PitchTrack.o :	\
	../../Reconstruction/PitchTrack.c
	$(CUR_CC_RECON) $(CC_OUT) $@  ../../Reconstruction/$(@F:.o=.c)

$(PROG2_OBJ)/error_handle.o :	\
	../../Reconstruction/error_handle.c
	$(CUR_CC_RECON) $(CC_OUT) $@  ../../Reconstruction/$(@F:.o=.c)


